#!/usr/bin/env perl

use strict;
use warnings;

use Test::More tests => 21;
use Test::NoWarnings;

use autodie;
use File::Temp qw(tempdir);
use File::Path qw(remove_tree);

BEGIN { use_ok('File::Rewrite', 'fileset') or BAIL_OUT('No fileset()'); }

my $target = tempdir();

sub testfileset {
	my ($name, $input, $params, $expect, $compare) = @_;

	my $filename = "$target/$name";
	open my $file, '>', $filename;
	print {$file} $input;
	close $file;

	my $result = eval { fileset($filename, @$params) };
	if (defined $result and defined $compare) {
		my $output = eval {
			local $/;
			open my $confirm, '<', $filename;
			readline $confirm;
		};
		if ($compare ne $output) {
			fail($name);
			diag(sprintf(
				'  Output mismatch (got %d of %d lines with return value %s)',
				$output =~ tr/\n//,
				$compare =~ tr/\n//,
				explain($result),
			));
			diag('         got: "', join('\n', split $/, $output ), '"');
			diag('    expected: "', join('\n', split $/, $compare), '"');
			return;
		}
	}
	is($result, $expect, $name);
}

my $BASICBODY = "replace$/noremove$/remove$/.$/";

testfileset('no match',      $BASICBODY,    ['no'      => undef], 0, $BASICBODY);
testfileset('remove string', $BASICBODY,    ['remove'  => undef], 1, "replace$/noremove$/.$/");
testfileset('remove empty',  "$/.$/ $/$/",  [''        => undef], 2, ".$/ $/");
testfileset('remove regex',  $BASICBODY,    [qr/^re/   => undef], 2, "noremove$/.$/");
testfileset('remove all',    $BASICBODY,    [qr/./     => undef], 4, '');

testfileset('add unconditionally', "$/0$/", [undef()   => '0'], 1, "$/0$/0$/");
testfileset('add string',     "foo$/",      ['bar'],            1, "foo$/bar$/");
testfileset('keep string',    "foo$/bar$/", ['foo'],            0, "foo$/bar$/");
testfileset('keep first',     "foo$/FOO$/foo$/", ['foo'],       1, "foo$/FOO$/");
testfileset('replace string', "$/0$/1$/2$/", ['0' => 3],        2, "$/1$/2$/3$/");
testfileset('keep regex',     "foo$/hi$/$/", [qr/./ => 'hi'],   1, "hi$/$/");

testfileset('add new array',  ".$/",        ['' => ['foo','.']],  2, ".$/foo$/.$/");
testfileset('replace by array', "$/foo$/.$/", ['' => ['foo','']], 3, "foo$/.$/foo$/$/");
testfileset('partial add',    "foo$/$/",    ['foo' => ['foo','.']], 1, "foo$/$/.$/");

my $SAMPLE2 = "$/foo$/and$/bar$/.$/";
testfileset('keep pair',      $SAMPLE2, [['foo','bar']], 0, $SAMPLE2);
testfileset('keep order',     $SAMPLE2, [['bar','foo']], 2, "$/and$/bar$/.$/foo$/");
testfileset('replace pair',   $SAMPLE2, [['bar','foo'] => ['fooo','barr']], 4, "$/and$/.$/fooo$/barr$/");
testfileset('mixed arrays',   $SAMPLE2, [['ignore',qr/a/] => ['bar','foo','.']], 3, "$/foo$/bar$/.$/foo$/.$/");

is(remove_tree($target), 19, 'no unexpected files');

